@@ -28,6 +28,7 @@
 
 // additional headers
 #include <cassert>
+#include <stdexcept>
 #include <sys/select.h>
 #include <unistd.h>
 #include <errno.h>
@@ -47,9 +48,12 @@ aiounicast_select::aiounicast_select
 		aiounicast(n_in, j_in, aio_default_scheduler_in, aio_default_timeout_in,
 			aio_is_authenticated_in, aio_is_encrypted_in)
 {
-	assert(j_in < n_in);
-	assert(n_in == fd_in_in.size());
-	assert(fd_in_in.size() == fd_out_in.size());
+	if (j_in >= n_in)
+		throw std::invalid_argument("aiounicast_select: j >= n");
+	if (fd_in_in.size() != n_in)
+		throw std::invalid_argument("aiounicast_select: |fd_in| != n");
+	if (fd_in_in.size() != fd_out_in.size())
+		throw std::invalid_argument("aiounicast_select: |fd_in| != |fd_out|");
 
 	// initialize scheduler
 	aio_schedule_current = 0, aio_schedule_buffer = 0;
@@ -75,7 +79,9 @@ aiounicast_select::aiounicast_select
 		if (maclen == 0)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_mac_get_algo_maclen() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_get_algo_maclen()" <<
+				" failed" << std::endl;
+			throw std::invalid_argument("aiounicast_select: bad MAC algo");
 		}
 	}
 	else
@@ -92,38 +98,44 @@ aiounicast_select::aiounicast_select
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_mac_open() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_open() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		memset(salt, 0, sizeof(salt));
-		err = gcry_kdf_derive(key_in[i].c_str(), key_in[i].length(), GCRY_KDF_PBKDF2, 
-			TMCG_GCRY_MD_ALGO, salt, sizeof(salt), 25000, sizeof(key), key);
+		err = gcry_kdf_derive(key_in[i].c_str(), key_in[i].length(),
+			GCRY_KDF_PBKDF2, TMCG_GCRY_MD_ALGO, salt, sizeof(salt), 25000,
+			sizeof(key), key);
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_kdf_derive() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_kdf_derive() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		err = gcry_mac_setkey(*mac_in[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_mac_setkey() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_setkey() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		err = gcry_mac_open(mac_out[i], TMCG_GCRY_MAC_ALGO, 0, NULL); 				
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_mac_open() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_open() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		err = gcry_mac_setkey(*mac_out[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_mac_setkey() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_setkey() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 	}
 
@@ -134,17 +146,23 @@ aiounicast_select::aiounicast_select
 		if (keylen == 0)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_cipher_get_algo_keylen() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_get_algo_keylen()" <<
+				" failed" << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		blklen = gcry_cipher_get_algo_blklen(TMCG_GCRY_ENC_ALGO);
 		if (blklen == 0)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_cipher_get_algo_blklen() failed" << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_get_algo_blklen()" <<
+				" failed" << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 	}
 	else
+	{
 		keylen = 0, blklen = 0;
+	}
 	for (size_t i = 0; aio_is_encrypted && (i < n_in); i++)
 	{
 		unsigned char salt[keylen];
@@ -154,51 +172,61 @@ aiounicast_select::aiounicast_select
 		gcry_cipher_hd_t *enc_in_hd = new gcry_cipher_hd_t();
 		gcry_cipher_hd_t *enc_out_hd = new gcry_cipher_hd_t();
 		enc_in.push_back(enc_in_hd), enc_out.push_back(enc_out_hd);
-		err = gcry_cipher_open(enc_in[i], TMCG_GCRY_ENC_ALGO, GCRY_CIPHER_MODE_CFB, 0); 				
+		err = gcry_cipher_open(enc_in[i], TMCG_GCRY_ENC_ALGO,
+			GCRY_CIPHER_MODE_CFB, 0); 				
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_cipher_open() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_open() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
-		memset(salt, 1, sizeof(salt)); // use a different salt to derive encryption key
-		err = gcry_kdf_derive(key_in[i].c_str(), key_in[i].length(), GCRY_KDF_PBKDF2, 
-			TMCG_GCRY_MD_ALGO, salt, sizeof(salt), 25000, sizeof(key), key);
+		// use a different salt to derive encryption key
+		memset(salt, 1, sizeof(salt));
+		err = gcry_kdf_derive(key_in[i].c_str(), key_in[i].length(),
+			GCRY_KDF_PBKDF2, TMCG_GCRY_MD_ALGO, salt, sizeof(salt), 25000,
+			sizeof(key), key);
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_kdf_derive() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_kdf_derive() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		err = gcry_cipher_setkey(*enc_in[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_cipher_setkey() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_setkey() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		iv_flag_in.push_back(false); // flag means: IV not yet received
-		err = gcry_cipher_open(enc_out[i], TMCG_GCRY_ENC_ALGO, GCRY_CIPHER_MODE_CFB, 0); 				
+		err = gcry_cipher_open(enc_out[i], TMCG_GCRY_ENC_ALGO,
+			GCRY_CIPHER_MODE_CFB, 0); 				
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_cipher_open() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_open() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		err = gcry_cipher_setkey(*enc_out[i], key, sizeof(key));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_cipher_setkey() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_setkey() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		gcry_create_nonce(iv, blklen); // unpredictable IV is sufficient
 		err = gcry_cipher_setiv(*enc_out[i], iv, sizeof(iv));
 		if (err)
 		{
 			aio_is_initialized = false;
-			std::cerr << "aiounicast_select: gcry_cipher_setiv() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_setiv() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
+			throw std::invalid_argument("aiounicast_select: libgcrypt failed");
 		}
 		iv_flag_out.push_back(false); // flag means: IV not yet sent
 		unsigned char *ivcopy = new unsigned char[blklen];
@@ -245,27 +273,31 @@ bool aiounicast_select::Send
 	}
 	else
 	{
-		std::cerr << "aiounicast_select(" << j << "): realsize does not fit" << std::endl;
+		std::cerr << "aiounicast_select(" << j << "):" <<
+			" realsize does not fit" << std::endl;
 		delete [] buf;
 		return false;
 	}
-	// We follow the Encrypt-then-Authenticate (EtA) paradigm, because it provides the
-	// best security properties with respect to the required 'secure channel'.
-	// Please note the scientific discussion on that topic,
-	// e.g., Mihir Bellare and Chanathip Namprempre: 'Authenticated Encryption: Relations
-	//       among notions and analysis of the generic composition paradigm', Advances in
-	//       Cryptology - ASIACRYPT 2000, LNCS 1976, pp. 531--545, 2000.
+	// We follow the Encrypt-then-Authenticate (EtA) paradigm, because it
+	// provides the best security properties with respect to the required
+	// 'secure channel'. Please note the scientific discussion on that topic,
+	// e.g., Mihir Bellare and Chanathip Namprempre: 'Authenticated Encryption:
+	//       Relations among notions and analysis of the generic composition
+	//       paradigm', Advances in Cryptology - ASIACRYPT 2000, LNCS 1976,
+	//       pp. 531--545, 2000.
 	gcry_error_t err;
-	// encrypt the content of write buffer (without delimiter) and send the IV to the receiver
+	// encrypt the content of write buffer (without delimiter) and send the
+	// IV to the receiver
 	if (aio_is_encrypted)
 	{
 		memmove(buf + 1, buf, realsize - 1);
 		buf[0] = '+'; // use plus-character as a non-zero prefix
-		err = gcry_cipher_encrypt(*enc_out[i_in], buf + 1, realsize - 1, NULL, 0);
+		err = gcry_cipher_encrypt(*enc_out[i_in], buf + 1, realsize - 1,
+			NULL, 0);
 		if (err)
 		{
-			std::cerr << "aiounicast_select: gcry_cipher_encrypt() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_cipher_encrypt() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
 			delete [] buf;
 			return false;
 		}
@@ -289,7 +321,8 @@ bool aiounicast_select::Send
 		}
 		else
 		{
-			std::cerr << "aiounicast_select(" << j << "): realsize does not fit" << std::endl;
+			std::cerr << "aiounicast_select(" << j << "):" <<
+				" realsize does not fit" << std::endl;
 			delete [] buf;
 			return false;
 		}
@@ -311,19 +344,28 @@ bool aiounicast_select::Send
 				retval = select((fd_out[i_in] + 1), NULL, &wfds, NULL, &tv);
 				if (retval < 0)
 				{
-					perror("aiounicast_select (select)");
-					delete [] buf;
-					return false;
+					if (errno == EINTR)
+					{
+						continue;
+					}
+					else
+					{
+						perror("aiounicast_select (select)");
+						delete [] buf;
+						return false;
+					}
 				}
 				if (retval == 0)
 					continue;
 				// write(2) -- ready for non-blocking write?
 				if (FD_ISSET(fd_out[i_in], &wfds))
 				{
-					ssize_t num = write(fd_out[i_in], iv_out[i_in] + realnum, blklen - realnum);
+					ssize_t num = write(fd_out[i_in], iv_out[i_in] + realnum,
+						blklen - realnum);
 					if (num < 0)
 					{
-						if ((errno == EAGAIN) || (errno == EWOULDBLOCK) || (errno == EINTR))
+						if ((errno == EAGAIN) || (errno == EWOULDBLOCK) ||
+							(errno == EINTR))
 						{
 							if (errno == EAGAIN)
 								perror("aiounicast_select (write)");
@@ -340,13 +382,17 @@ bool aiounicast_select::Send
 					realnum += num;
 				}
 				else
-					std::cerr << "WARNING: aiounicast_select FD_ISSET not true" << std::endl;
+				{
+					std::cerr << "WARNING: aiounicast_select FD_ISSET" <<
+						" not true" << std::endl;
+				}
 			}
 			while ((realnum < blklen) && (time(NULL) < (entry_time + timeout)));
 			// timeout occurred?
 			if (realnum < blklen)
 			{
-				std::cerr << "aiounicast_select(" << j << "): IV send timeout for " << i_in << std::endl;
+				std::cerr << "aiounicast_select(" << j << "):" <<
+					" IV send timeout for " << i_in << std::endl;
 				delete [] buf;
 				return false;
 			}
@@ -360,8 +406,8 @@ bool aiounicast_select::Send
 		err = gcry_mac_write(*mac_out[i_in], buf, realsize);
 		if (err)
 		{
-			std::cerr << "aiounicast_select: gcry_mac_write() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_write() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
 			delete [] buf;
 			return false;
 		}
@@ -382,9 +428,16 @@ bool aiounicast_select::Send
 		retval = select((fd_out[i_in] + 1), NULL, &wfds, NULL, &tv);
 		if (retval < 0)
 		{
-			perror("aiounicast_select (select)");
-			delete [] buf;
-			return false;
+			if (errno == EINTR)
+			{
+				continue;
+			}
+			else
+			{
+				perror("aiounicast_select (select)");
+				delete [] buf;
+				return false;
+			}
 		}
 		if (retval == 0)
 			continue;
@@ -394,7 +447,8 @@ bool aiounicast_select::Send
 			ssize_t num = write(fd_out[i_in], buf + realnum, realsize - realnum);
 			if (num < 0)
 			{
-				if ((errno == EAGAIN) || (errno == EWOULDBLOCK) || (errno == EINTR))
+				if ((errno == EAGAIN) || (errno == EWOULDBLOCK) ||
+					(errno == EINTR))
 				{
 					if (errno == EAGAIN)
 						perror("aiounicast_select (write)");
@@ -411,14 +465,18 @@ bool aiounicast_select::Send
 			realnum += num;
 		}
 		else
-			std::cerr << "WARNING: aiounicast_select(" << j << ") FD_ISSET not true" << std::endl;
+		{
+			std::cerr << "WARNING: aiounicast_select(" << j << ") FD_ISSET" <<
+				" not true" << std::endl;
+		}
 	}
 	while ((realnum < realsize) && (time(NULL) < (entry_time + timeout)));
 	delete [] buf;
 	// timeout occurred?
 	if (realnum < realsize)
 	{
-		std::cerr << "aiounicast_select(" << j << "): send timeout for " << i_in << std::endl;
+		std::cerr << "aiounicast_select(" << j << "):" <<
+			" send timeout for " << i_in << std::endl;
 		return false;
 	}
 	if (aio_is_authenticated)
@@ -429,16 +487,16 @@ bool aiounicast_select::Send
 		err = gcry_mac_read(*mac_out[i_in], macbuf, &macbuflen);
 		if (err)
 		{
-			std::cerr << "aiounicast_select: gcry_mac_read() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_read() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
 			delete [] macbuf;
 			return false;
 		}
 		err = gcry_mac_reset(*mac_out[i_in]);
 		if (err)
 		{
-			std::cerr << "aiounicast_select: gcry_mac_reset() failed" << std::endl;
-			std::cerr << gcry_strerror(err) << std::endl;
+			std::cerr << "aiounicast_select: gcry_mac_reset() failed" <<
+				std::endl << gcry_strerror(err) << std::endl;
 			delete [] macbuf;
 			return false;
 		}
@@ -457,19 +515,28 @@ bool aiounicast_select::Send
 			retval = select((fd_out[i_in] + 1), NULL, &wfds, NULL, &tv);
 			if (retval < 0)
 			{
-				perror("aiounicast_select (select)");
-				delete [] macbuf;
-				return false;
+				if (errno == EINTR)
+				{
+					continue;
+				}
+				else
+				{
+					perror("aiounicast_select (select)");
+					delete [] macbuf;
+					return false;
+				}
 			}
 			if (retval == 0)
 				continue;
 			// write(2) -- ready for non-blocking write?
 			if (FD_ISSET(fd_out[i_in], &wfds))
 			{
-				ssize_t num = write(fd_out[i_in], macbuf + realnum, macbuflen - realnum);
+				ssize_t num = write(fd_out[i_in], macbuf + realnum,
+					macbuflen - realnum);
 				if (num < 0)
 				{
-					if ((errno == EAGAIN) || (errno == EWOULDBLOCK) || (errno == EINTR))
+					if ((errno == EAGAIN) || (errno == EWOULDBLOCK) ||
+						(errno == EINTR))
 					{
 						if (errno == EAGAIN)
 							perror("aiounicast_select (write)");
@@ -486,18 +553,21 @@ bool aiounicast_select::Send
 				realnum += num;
 			}
 			else
-				std::cerr << "WARNING: aiounicast_select FD_ISSET not true" << std::endl;
+			{
+				std::cerr << "WARNING: aiounicast_select FD_ISSET not true" <<
+					std::endl;
+			}
 		}
 		while ((realnum < macbuflen) && (time(NULL) < (entry_time + timeout)));
 		delete [] macbuf;
 		// timeout occurred?
 		if (realnum < macbuflen)
 		{
-			std::cerr << "aiounicast_select(" << j << "): MAC send timeout for " << i_in << std::endl;
+			std::cerr << "aiounicast_select(" << j << "):" <<
+				" MAC send timeout for " << i_in << std::endl;
 			return false;
 		}
 	}
-//std::cerr << "send(" << j << ") to " << i_in << ": m = " << m << std::endl;
 	return true;
 }
 
@@ -570,18 +640,22 @@ bool aiounicast_select::Receive
 					}
 				}
 				// process the buffer
-				if (newline_found && ((buf_ptr[i_out] - newline_ptr - 1) >= maclen))
+				if (newline_found &&
+					((buf_ptr[i_out] - newline_ptr - 1) >= maclen))
 				{
-					char *tmp = new char[newline_ptr + 1]; // allocate at least one char
-					unsigned char *mac = new unsigned char[maclen + 1]; // allocate at least one char, if maclen == 0
-					memset(tmp, 0, newline_ptr + 1);
+					// allocate at least one char
+					size_t tmplen = newline_ptr + 1;
+					char *tmp = new char[tmplen];
+					// allocate at least one char, even if maclen == 0
+					unsigned char *mac = new unsigned char[maclen + 1];
+					memset(tmp, 0, tmplen);
 					memset(mac, 0, maclen);
 					if (newline_ptr > 0)
 						memcpy(tmp, buf_in[i_out], newline_ptr);
 					if (maclen > 0)
-						memcpy(mac, buf_in[i_out] + newline_ptr + 1, maclen);
+						memcpy(mac, buf_in[i_out] + tmplen, maclen);
 					// adjust buffer (copy remaining characters)
-					unsigned char *wptr = buf_in[i_out] + newline_ptr + 1 + maclen;
+					unsigned char *wptr = buf_in[i_out] + tmplen + maclen;
 					size_t wnum = buf_ptr[i_out] - newline_ptr - 1 - maclen;
 					if (wnum > 0)
 						memmove(buf_in[i_out], wptr, wnum);
@@ -595,8 +669,9 @@ bool aiounicast_select::Receive
 						err = gcry_mac_write(*mac_in[i_out], tmp, newline_ptr);
 						if (err)
 						{
-							std::cerr << "aiounicast_select: gcry_mac_write() failed" << std::endl;
-							std::cerr << gcry_strerror(err) << std::endl;
+							std::cerr << "aiounicast_select:" <<
+								" gcry_mac_write() failed" << std::endl <<
+								gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
@@ -605,8 +680,9 @@ bool aiounicast_select::Receive
 						err = gcry_mac_write(*mac_in[i_out], &delim, 1);
 						if (err)
 						{
-							std::cerr << "aiounicast_select: gcry_mac_write() failed" << std::endl;
-							std::cerr << gcry_strerror(err) << std::endl;
+							std::cerr << "aiounicast_select:" <<
+								" gcry_mac_write() failed" << std::endl <<
+								gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
@@ -614,71 +690,79 @@ bool aiounicast_select::Receive
 						err = gcry_mac_verify(*mac_in[i_out], mac, maclen);
 						if (err)
 						{
-							std::cerr << "aiounicast_select: gcry_mac_verify() for " << j << " from " << i_out << " failed" << std::endl;
-							std::cerr << gcry_strerror(err) << std::endl;
+							std::cerr << "aiounicast_select:" <<
+								" gcry_mac_verify() for " << j << " from " <<
+								i_out << " failed" << std::endl <<
+								gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
 						err = gcry_mac_reset(*mac_in[i_out]);
 						if (err)
 						{
-							std::cerr << "aiounicast_select: gcry_mac_reset() failed" << std::endl;
-							std::cerr << gcry_strerror(err) << std::endl;
+							std::cerr << "aiounicast_select:" <<
+								" gcry_mac_reset() failed" << std::endl <<
+								gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
 					}
-					// convert and decrypt the corresponding part of the read buffer
+					// convert and decrypt the corresponding part of read buffer
 					if (aio_is_encrypted)
 					{
 						mpz_t encval;
 						mpz_init(encval);
 						if (mpz_set_str(encval, tmp, TMCG_MPZ_IO_BASE) < 0)
 						{
-							std::cerr << "aiounicast_select: mpz_set_str() for encval failed" << std::endl;
+							std::cerr << "aiounicast_select: mpz_set_str()" <<
+								" for encval failed" << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
 						size_t realsize = 0;
-						memset(tmp, 0, newline_ptr + 1); // clear tmp buffer
+						memset(tmp, 0, tmplen); // clear tmp buffer
 						mpz_export(tmp, &realsize, 1, 1, 1, 0, encval);
 						if (realsize == 0)
 						{
-							std::cerr << "aiounicast_select: mpz_export() failed for " << encval << std::endl;
+							std::cerr << "aiounicast_select: mpz_export()" <<
+								" failed for " << encval << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
 						mpz_clear(encval);
 						if (tmp[0] != '+')
 						{
-							std::cerr << "aiounicast_select: no prefix found" << std::endl;
+							std::cerr << "aiounicast_select: no prefix" <<
+								" found" << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
 						gcry_error_t err;
-						err = gcry_cipher_decrypt(*enc_in[i_out], tmp + 1, realsize - 1, NULL, 0);
+						err = gcry_cipher_decrypt(*enc_in[i_out], tmp + 1,
+							realsize - 1, NULL, 0);
 						if (err)
 						{
-							std::cerr << "aiounicast_select: gcry_cipher_decrypt() failed" << std::endl;
-							std::cerr << gcry_strerror(err) << std::endl;
+							std::cerr << "aiounicast_select:" <<
+								" gcry_cipher_decrypt() failed" << std::endl <<
+								gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
 						}
 						numDecrypted += (realsize - 1);
 						memmove(tmp, tmp + 1, realsize - 1); // remove prefix
-						tmp[realsize-1] = 0x00; // append a string delimiter
+						tmp[realsize-1] = 0x00; // append a c-string delimiter
 					}
 					// extract value of m
 					if (mpz_set_str(m, tmp, TMCG_MPZ_IO_BASE) < 0)
 					{
-						std::cerr << "aiounicast_select: mpz_set_str() for m from " << i_out << " failed" << std::endl;
+						std::cerr << "aiounicast_select: mpz_set_str() for" <<
+							" m from " << i_out << " failed" << std::endl;
 						delete [] tmp, delete [] mac;
 						return false;
 					}
 					delete [] tmp, delete [] mac;
 					if (aio_is_encrypted)
 						mpz_sub(m, m, aio_hide_length);
-//std::cerr << "receive(" << j << ") from " << i_out << ": m = " << m << std::endl;
 					return true;
 				}
 				// no delimiter found; invalidate buffer flag
@@ -701,8 +785,15 @@ bool aiounicast_select::Receive
 				retval = select((fd_in[i_out] + 1), &rfds, NULL, NULL, &tv);
 				if (retval < 0)
 				{
-					perror("aiounicast_select (select)");
-					return false;
+					if (errno == EINTR)
+					{
+						continue;
+					}
+					else
+					{
+						perror("aiounicast_select (select)");
+						return false;
+					}
 				}
 				if (retval == 0)
 					continue;
@@ -719,28 +810,31 @@ bool aiounicast_select::Receive
 					if (num == 0)
 					{
 						// got EOF
-						std::cerr << "aiounicast_select(" << j << "): got EOF for " << i_out << std::endl;
-						fd_in.erase(i_out); // erase corresponding input file descriptor
-						fd_out.erase(i_out); // erase corresponding output file descriptor
+						std::cerr << "aiounicast_select(" << j << "):" <<
+							" got EOF for " << i_out << std::endl;
+						fd_in.erase(i_out); // erase input file descriptor
+						fd_out.erase(i_out); // erase output file descriptor
 						continue;
 					}
 					buf_ptr[i_out] += num;
 					numRead += num;
 					if (aio_is_encrypted)
 					{
-						// take the first blklen bytes from sender as IV for cipher
+						// take first blklen bytes from sender as IV for cipher
 						if (!iv_flag_in[i_out] && (buf_ptr[i_out] >= blklen))
 						{
 							gcry_error_t err;							
-							err = gcry_cipher_setiv(*enc_in[i_out], buf_in[i_out], blklen);
+							err = gcry_cipher_setiv(*enc_in[i_out],
+								buf_in[i_out], blklen);
 							if (err)
 							{
 								aio_is_initialized = false;
-								std::cerr << "aiounicast_select: gcry_cipher_setiv() failed" << std::endl;
-								std::cerr << gcry_strerror(err) << std::endl;
+								std::cerr << "aiounicast_select:" <<
+									" gcry_cipher_setiv() failed" << std::endl <<
+									gcry_strerror(err) << std::endl;
 							}
 							iv_flag_in[i_out] = true; // IV is set
-							num = buf_ptr[i_out] - blklen; // number of remaining bytes
+							num = buf_ptr[i_out] - blklen; // # of remaining bytes
 							// remove IV from the read buffer
 							memmove(buf_in[i_out], buf_in[i_out] + blklen, num);
 							buf_ptr[i_out] = num;
@@ -752,17 +846,26 @@ bool aiounicast_select::Receive
 						buf_flag[i_out] = true;
 				}
 				else
-					std::cerr << "WARNING: aiounicast_select FD_ISSET not true" << std::endl;
+				{
+					std::cerr << "WARNING: aiounicast_select FD_ISSET" <<
+						" not true" << std::endl;
+				}
 			}
 			else
-				std::cerr << "WARNING: aiounicast_select read buffer exceeded" << std::endl;
+			{
+				std::cerr << "WARNING: aiounicast_select read buffer" <<
+					" exceeded" << std::endl;
+			}
 		}
 	}
 	while (time(NULL) < (entry_time + timeout));
 	if (scheduler != aio_scheduler_direct)
 		i_out = n; // timeout for some (unknown) parties
 	else
-		std::cerr << "aiounicast_select(" << j << "): timeout for " << i_out << std::endl;
+	{
+		std::cerr << "aiounicast_select(" << j << "):" <<
+			" timeout for " << i_out << std::endl;
+	}
 	return false;
 }
 
